package org.edg.wl.dgas.dgassim.analysis.math;

import org.edg.wl.dgas.dgassim.analysis.util.Result;
import java.util.ArrayList;

/**
   a class for simple statistical analysis of sets of experimental or
   simulation results.
 */
public class StatisticalAnalyzer {

    /**
       computes a mean (average) value.
       @return mean value.
       @param resultSet array of
       {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
     */
    public static double meanValue(Result[] resultSet) {
	double total = 0;

	for (int num = 0; num < resultSet.length; num++)
	    total += resultSet[num].getDoubleValue();

	return (total / resultSet.length);
    }

    /**
       computes a mean (average) value.
       @return mean value.
       @param resultSet <b>ArrayList</b> of
       {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
       @throws ClassCastException if the ArrayList doesn't contain objects
       of type {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
     */
    public static double meanValue(ArrayList resultSet)
	throws ClassCastException {

	return meanValue( (Result[]) resultSet.toArray() );
    }


    /**
       computes a mean (average) absolute deviation from the mean value.
       @return mean absolute deviation.
       @param resultSet array of
       {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
     */
    public static double meanDeviation(Result[] resultSet) {
	double meanV = meanValue(resultSet);

	double totalDev = 0;
	for (int num = 0; num < resultSet.length; num++)
	    totalDev += Math.abs( resultSet[num].getDoubleValue() - meanV );

	return (totalDev / resultSet.length);
    }


    /**
       computes a mean (average) absolute deviation from the mean value.
       @return mean absolute deviation.
       @param resultSet array of
       {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
       @throws ClassCastException if the ArrayList doesn't contain objects
       of type {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
    */
    public static double meanDeviation(ArrayList resultSet)
	throws ClassCastException {

	return meanDeviation( (Result[]) resultSet.toArray() );
    }


    /**
       computes a standard deviation from the mean value.
       @return standard deviation.
       @param resultSet array of
       {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
     */
    public static double standardDeviation(Result[] resultSet) {
	double meanV = meanValue(resultSet);

	double totalSqDev = 0;
	for (int num = 0; num < resultSet.length; num++)
	    totalSqDev += Math.pow( resultSet[num].getDoubleValue() - meanV,
				    2.0 );

	return Math.sqrt( totalSqDev / (resultSet.length-1) );
    }


    /**
       computes a standard deviation from the mean value.
       @return standard deviation.
       @param resultSet array of
       {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
       @throws ClassCastException if the ArrayList doesn't contain objects
       of type {@link org.edg.wl.dgas.dgassim.analysis.util.Result Result}s.
    */
    public static double standardDeviation(ArrayList resultSet)
	throws ClassCastException {

	return standardDeviation( (Result[]) resultSet.toArray() );
    }

}
