package org.edg.wl.dgas.dgassim.workload.rb;

import java.util.ArrayList;
import org.edg.wl.dgas.dgassim.*;
import org.edg.wl.dgas.dgassim.fabric.*;
import org.edg.wl.dgas.dgassim.monitor.*;

/**
   The <code><b>CEAttribMatchmaker</b></code> can be used by a
   {@link org.edg.wl.dgas.dgassim.workload.rb.ResourceBroker ResourceBroker}.
   It uses the  {@link org.edg.wl.dgas.dgassim.Job#matchCE Job.matchCE()}
   method to let the given {@link org.edg.wl.dgas.dgassim.Job Job} decide which
   {@link org.edg.wl.dgas.dgassim.fabric.ComputingElement ComputingElement}s'
   characteristics match. No other matchmaking information, e.g. data acces
   time, is considered.
 */
public class CEAttribMatchmaker implements Matchmaker {

    private GIS gis;

    /**
       creates an instance.
       @param gis the {@link org.edg.wl.dgas.dgassim.monitor.GIS GIS}
       (Grid Information System) instance to which all
       {@link org.edg.wl.dgas.dgassim.fabric.ComputingElement ComputingElement}s
       are registered.
     */
    public CEAttribMatchmaker(GIS gis) {
	this.gis = gis;
    }

    /**
       executes the matchmaking considering only the
       {@link org.edg.wl.dgas.dgassim.fabric.ComputingElement ComputingElement}s'
       characteristics by using the 
       {@link org.edg.wl.dgas.dgassim.Job#matchCE Job.matchCE()} method.
       @return An <code><b>ArrayList</b></code> of references to all matching
       {@link org.edg.wl.dgas.dgassim.fabric.ComputingElement ComputingElement}s.
       @param job the {@link org.edg.wl.dgas.dgassim.Job Job} for which to
       get a list of matching
       {@link org.edg.wl.dgas.dgassim.fabric.ComputingElement ComputingElement}s.
     */
    public ArrayList getMatchingCEList(Job job) {
	ArrayList ces = gis.getCEList();
	ArrayList matchingCEs = new ArrayList();

	ComputingElement ce = null;

	for (int i = 0; i < ces.size(); i++) {

	    ce = (ComputingElement)ces.get(i);

	    
	    if (job.matchCE(ce))
		matchingCEs.add(ce); // CE matches job requirements
	}

	return matchingCEs;

    }

}
